import os
import uuid

# For python 2 support
# from io import open

from abejacli.session import (
    generate_retry_session,
    generate_user_session,
)
from abejacli.config import (
    PLATFORM_BASE_URL,
    HTTP_READ_CHUNK_SIZE,
)
from abejacli.datalake.process_file_job import (
    INITIALIZE_REPORT,
    PROGRESS_REPORT,
    FINISH_REPORT,
    RAISE_ERROR
)


def download_job(channel_id, file_info, report_queue, options):
    """
    download files until consuming all items in file queue

    :param channel_id: channel identifier
    :param file_info: information of file to download
    :param report_queue: queue to report progress for each file
    :param options: option including target download directory
    :return:
    """

    publisher_id = uuid.uuid4().hex
    download_dir = options.get('download_dir', '.')
    file_id, download_url, file_name = file_info

    try:
        session = generate_retry_session()
        download_stream_res = session.get(download_url, stream=True, timeout=10)
        if download_stream_res.status_code == 403:
            # request to update download url
            user_session = generate_user_session()
            url = "{}/channels/{}/{}".format(PLATFORM_BASE_URL, channel_id, file_id)
            res = user_session.get(url, timeout=10)
            res.raise_for_status()
            file_info = res.json()
            download_url = file_info.get('download_url')
            download_stream_res = session.get(download_url, stream=True, timeout=10)
        download_stream_res.raise_for_status()
        total_size = int(download_stream_res.headers.get('content-length', 0))

        initialize_options = {
            'file_name': file_name,
            'total': total_size,
        }
        report_queue.put((INITIALIZE_REPORT, publisher_id, 0, initialize_options))
        download_path = os.path.join(download_dir, file_id)
        with open(download_path, 'wb') as f:
            for chunk in download_stream_res.iter_content(chunk_size=HTTP_READ_CHUNK_SIZE):
                # update tqdm progress bar with chunk data size
                report_queue.put((PROGRESS_REPORT, publisher_id, len(chunk), None))
                f.write(chunk)
        result_options = {
            'source': file_id,
            'destination': download_path,
        }
        report_queue.put((FINISH_REPORT, publisher_id, 0, result_options))
    except:
        options = {
            'source': file_id,
            'error': 'Failed to download {} of channel_id {}'.format(file_id, channel_id)
        }
        report_queue.put((RAISE_ERROR, publisher_id, 0, options))
